import { EquipmentSlot, GameMode, ItemDurabilityComponent, ItemEnchantableComponent, ItemStack, } from "@minecraft/server";
import { playerEquipmentManager } from "../../Player/playerEquipmentManager";
import { itemJetpackManager } from "../../Items/itemJetpackManager";
import { playerManager } from "../../Player/playerManager";
export var BlockOilTankStates;
(function (BlockOilTankStates) {
    BlockOilTankStates["empty"] = "empty";
    BlockOilTankStates["lava"] = "lava";
    BlockOilTankStates["obsidian"] = "obsidian";
    BlockOilTankStates["oil"] = "oil";
})(BlockOilTankStates || (BlockOilTankStates = {}));
export var CustomBucketTypes;
(function (CustomBucketTypes) {
    CustomBucketTypes["oil_bucket"] = "eddsjetpacks:oil_bucket";
})(CustomBucketTypes || (CustomBucketTypes = {}));
const heatBlocks = [
    "minecraft:torch",
    "minecraft:soul_torch",
    "minecraft:campfire",
    "minecraft:soul_campfire",
    "minecraft:lava",
    "minecraft:flowing_lava"
];
const bucketIDs = [
    "minecraft:lava_bucket",
    "minecraft:water_bucket",
    "minecraft:bucket",
    CustomBucketTypes.oil_bucket
];
export class blockCustomComponentManager {
    static customComponents = [
        {
            id: "eddsjetpacks:oil_tank",
            code: {
                onPlayerInteract: (data) => {
                    const { dimension, block, player, face } = data;
                    const state = block.permutation.getState("eddsjetpacks:oil_state");
                    if (!player || !player.isValid())
                        return;
                    const slot = playerEquipmentManager.getSlot(player, EquipmentSlot.Mainhand);
                    if (!slot)
                        return;
                    const item = slot.getItem();
                    if (!item)
                        return;
                    const typeId = item.typeId;
                    const jetpackData = itemJetpackManager.jetpacks.find((f) => f.emptyJetpack && f.emptyJetpack.itemID == typeId);
                    if (!bucketIDs.includes(typeId) &&
                        !jetpackData)
                        return;
                    if (jetpackData) {
                        if (!jetpackData)
                            return;
                        const emptyJetpackData = jetpackData.emptyJetpack;
                        if (!emptyJetpackData)
                            return;
                        const jetpackItem = new ItemStack(jetpackData.jetpackID, 1);
                        if (emptyJetpackData.transferDurability) {
                            const lastDurComp = item.getComponent(ItemDurabilityComponent.componentId);
                            const newDurComp = jetpackItem.getComponent(ItemDurabilityComponent.componentId);
                            newDurComp.damage = lastDurComp.damage;
                        }
                        if (emptyJetpackData.transferEnchantments) {
                            const lastEnchantComp = item.getComponent(ItemEnchantableComponent.componentId);
                            if (lastEnchantComp) {
                                const lastEnchants = lastEnchantComp.getEnchantments();
                                const newEnchantComp = jetpackItem.getComponent(ItemEnchantableComponent.componentId);
                                if (newEnchantComp) {
                                    newEnchantComp.addEnchantments(lastEnchants);
                                }
                            }
                        }
                        if (item.nameTag) {
                            jetpackItem.nameTag = item.nameTag;
                        }
                        jetpackItem.setLore(itemJetpackManager.getLore(jetpackData.maxFuel, jetpackData.maxFuel));
                        jetpackItem.setDynamicProperty("fuel", jetpackData.maxFuel);
                        slot.setItem(jetpackItem);
                        block.setPermutation(block.permutation.withState("eddsjetpacks:oil_state", BlockOilTankStates.empty));
                        dimension.playSound("bucket.fill_lava", block.center());
                    }
                    else {
                        switch (typeId) {
                            case "minecraft:lava_bucket":
                                if (state != BlockOilTankStates.empty)
                                    break;
                                const newBucket = new ItemStack("minecraft:bucket", 1);
                                if (player.getGameMode() != GameMode.creative)
                                    slot.setItem(newBucket);
                                block.setPermutation(block.permutation.withState("eddsjetpacks:oil_state", BlockOilTankStates.lava));
                                dimension.playSound("bucket.empty_lava", block.center());
                                break;
                            case "minecraft:bucket":
                                if (state == BlockOilTankStates.lava) {
                                    const newLavaBucket = new ItemStack("minecraft:lava_bucket", 1);
                                    if (item.amount > 1) {
                                        item.amount = item.amount - 1;
                                        if (player.getGameMode() != GameMode.creative)
                                            slot.setItem(item);
                                        playerManager.giveItem(player, newLavaBucket);
                                    }
                                    else if (player.getGameMode() != GameMode.creative)
                                        slot.setItem(newLavaBucket);
                                    block.setPermutation(block.permutation.withState("eddsjetpacks:oil_state", BlockOilTankStates.empty));
                                    dimension.playSound("bucket.fill_lava", block.center());
                                }
                                else if (state == BlockOilTankStates.oil) {
                                    const newOilBucket = new ItemStack("eddsjetpacks:oil_bucket", 1);
                                    if (item.amount > 1) {
                                        item.amount = item.amount - 1;
                                        slot.setItem(item);
                                        if (player.getGameMode() != GameMode.creative)
                                            playerManager.giveItem(player, newOilBucket);
                                    }
                                    else if (player.getGameMode() != GameMode.creative)
                                        slot.setItem(newOilBucket);
                                    block.setPermutation(block.permutation.withState("eddsjetpacks:oil_state", BlockOilTankStates.empty));
                                    dimension.playSound("bucket.fill_lava", block.center());
                                }
                                else
                                    break;
                                break;
                            case "minecraft:water_bucket":
                                if (state != BlockOilTankStates.lava)
                                    return;
                                const newWaterBucket = new ItemStack("minecraft:bucket", 1);
                                if (player.getGameMode() != GameMode.creative)
                                    slot.setItem(newWaterBucket);
                                block.setPermutation(block.permutation.withState("eddsjetpacks:oil_state", BlockOilTankStates.obsidian));
                                dimension.playSound("bucket.empty_water", block.center());
                                break;
                            case CustomBucketTypes.oil_bucket:
                                if (state != BlockOilTankStates.empty)
                                    break;
                                const newRegularBucket = new ItemStack("minecraft:bucket", 1);
                                if (player.getGameMode() != GameMode.creative)
                                    slot.setItem(newRegularBucket);
                                block.setPermutation(block.permutation.withState("eddsjetpacks:oil_state", BlockOilTankStates.oil));
                                dimension.playSound("bucket.empty_lava", block.center());
                                break;
                        }
                    }
                },
                onRandomTick: (data) => {
                    const { block, dimension } = data;
                    const state = block.permutation.getState("eddsjetpacks:oil_state");
                    if (state != BlockOilTankStates.obsidian)
                        return;
                    let blockBelow = undefined;
                    try {
                        blockBelow = block.below(1);
                    }
                    catch { }
                    if (blockBelow == undefined || !heatBlocks.includes(blockBelow.typeId))
                        return;
                    block.setPermutation(block.permutation.withState("eddsjetpacks:oil_state", BlockOilTankStates.oil));
                },
            },
        },
    ];
    static load(data) {
        //a
        for (const comp of this.customComponents) {
            data.blockComponentRegistry.registerCustomComponent(comp.id, comp.code);
        }
    }
}
