import { EquipmentSlot, ItemDurabilityComponent, ItemEnchantableComponent, ItemStack, system, world, } from "@minecraft/server";
import { playerEquipmentManager } from "../Player/playerEquipmentManager";
import { playerManager } from "../Player/playerManager";
export class itemJetpackManager {
    static usingJetpackTag = "eddsjetpacks_using_item";
    static jetpackItemID = "eddsjetpacks:jetpack_controller";
    static jetpacks = [
        {
            jetpackID: "eddsjetpacks:jetpack",
            maxFuel: 50,
            speed: {
                forward: 0.5,
                upwards: 0.7
            },
            fuelConsumption: 33.333,
            emptyJetpack: {
                itemID: "eddsjetpacks:jetpack_empty",
                transferDurability: true,
                transferEnchantments: true
            }
        },
        {
            jetpackID: "eddsjetpacks:golden_jetpack",
            maxFuel: 75,
            speed: {
                forward: 1,
                upwards: 0.7
            },
            fuelConsumption: 34.333,
            emptyJetpack: {
                itemID: "eddsjetpacks:golden_jetpack_empty",
                transferDurability: true,
                transferEnchantments: true
            }
        },
        {
            jetpackID: "eddsjetpacks:obsidian_jetpack",
            maxFuel: 100,
            speed: {
                forward: 0.3,
                upwards: 0.625
            },
            fuelConsumption: 30,
            emptyJetpack: {
                itemID: "eddsjetpacks:obsidian_jetpack_empty",
                transferDurability: true,
                transferEnchantments: true
            }
        },
    ];
    static initialTick() {
        for (const player of world.getAllPlayers()) {
            this.tick(player);
            system.runTimeout(() => {
                if (player.isValid())
                    this.tick(player);
            }, 5);
        }
    }
    static tick(player) {
        const mainhand = playerEquipmentManager.getSlot(player, EquipmentSlot.Mainhand);
        if (!mainhand)
            return;
        const mainhandItem = mainhand.getItem();
        if (!mainhandItem || mainhandItem.typeId != "eddsjetpacks:jetpack_controller")
            return;
        const slot = playerEquipmentManager.getSlot(player, EquipmentSlot.Chest);
        if (!slot)
            return;
        const item = slot.getItem();
        if (!item)
            return;
        const jetpackData = this.getJetpackData(item.typeId);
        if (!jetpackData)
            return;
        const fuel = this.getFuel(item, slot, jetpackData.maxFuel);
        if (fuel <= 0) {
            this.setEmptyJetpack(player, jetpackData, item, slot);
        }
        else {
            if (player.hasTag(this.usingJetpackTag)) {
                const viewDir = player.getViewDirection();
                player.applyKnockback(viewDir.x, viewDir.z, jetpackData.speed.forward * this.getKnockbackMultiplier(player), jetpackData.speed.upwards * this.getKnockbackMultiplier(player));
                player.playSound("eddsjetpacks:jetpack_fly", { volume: 0.5 });
                this.reduceFuelTick(player, jetpackData.fuelConsumption);
                if (this.reduceFuelCheck(player)) {
                    this.setFuel(item, slot, fuel - 1, jetpackData.maxFuel);
                    playerManager.stopSound(player, "armor.equip_generic");
                }
            }
            else if (!player.isOnGround) {
                player.addEffect("slow_falling", 20, { showParticles: false });
                this.reduceFuelTick(player, jetpackData.fuelConsumption * 0.1);
                if (this.reduceFuelCheck(player)) {
                    this.setFuel(item, slot, fuel - 1, jetpackData.maxFuel);
                    playerManager.stopSound(player, "armor.equip_generic");
                }
            }
            if (!player.hasTag(this.usingJetpackTag))
                return;
        }
    }
    static getKnockbackMultiplier(player) {
        let multiplier = 1.0;
        const slots = [playerEquipmentManager.getSlot(player, EquipmentSlot.Head), playerEquipmentManager.getSlot(player, EquipmentSlot.Legs), playerEquipmentManager.getSlot(player, EquipmentSlot.Feet)];
        for (const slot of slots) {
            const item = slot.getItem();
            if (!item)
                continue;
            if (!item.typeId.includes("netherite"))
                continue;
            multiplier = multiplier + 0.1;
        }
        return multiplier;
    }
    static reduceFuelCheck(player) {
        const jetpackTick = this.getjetpackFuelTick(player);
        if (jetpackTick <= 0)
            player.setDynamicProperty("jetpackFuelTick", 100 + jetpackTick);
        if (jetpackTick <= 0) {
            return true;
        }
        else
            return false;
    }
    static reduceFuelTick(player, amount) {
        player.setDynamicProperty("jetpackFuelTick", this.getjetpackFuelTick(player) - amount);
    }
    static getjetpackFuelTick(player) {
        if (!player.getDynamicProperty("jetpackFuelTick")) {
            player.setDynamicProperty("jetpackFuelTick", 0);
        }
        return player.getDynamicProperty("jetpackFuelTick");
    }
    static setEmptyJetpack(player, jetpackData, item, slot) {
        if (jetpackData.emptyJetpack == undefined) {
            console.warn("no empty jetpack data found");
            return;
        }
        const emptyJetpackData = jetpackData.emptyJetpack;
        const emptyJetpackItem = new ItemStack(emptyJetpackData.itemID, 1);
        if (emptyJetpackData.transferDurability) {
            const lastDurComp = item.getComponent(ItemDurabilityComponent.componentId);
            const newDurComp = emptyJetpackItem.getComponent(ItemDurabilityComponent.componentId);
            newDurComp.damage = lastDurComp.damage;
        }
        if (emptyJetpackData.transferEnchantments) {
            const lastEnchantComp = item.getComponent(ItemEnchantableComponent.componentId);
            if (lastEnchantComp) {
                const lastEnchants = lastEnchantComp.getEnchantments();
                const newEnchantComp = emptyJetpackItem.getComponent(ItemEnchantableComponent.componentId);
                if (newEnchantComp) {
                    newEnchantComp.addEnchantments(lastEnchants);
                }
            }
        }
        if (item.nameTag) {
            emptyJetpackItem.nameTag = item.nameTag;
        }
        emptyJetpackItem.setLore(this.getLore(0, jetpackData.maxFuel));
        slot.setItem(emptyJetpackItem);
    }
    static getFuel(item, ContainerSlot, maxFuel) {
        if (item.getDynamicProperty("fuel") == undefined) {
            item.setLore(this.getLore(maxFuel, maxFuel));
            item.setDynamicProperty("fuel", maxFuel);
            ContainerSlot.setItem(item);
        }
        return (item.getDynamicProperty("fuel"));
    }
    static getLore(fuel, maxFuel) {
        return [`§r§fFuel: ${fuel} / ${maxFuel}`];
    }
    static setFuel(item, ContainerSlot, fuel, maxFuel) {
        item.setDynamicProperty("fuel", fuel);
        item.setLore(this.getLore(fuel, maxFuel));
        ContainerSlot.setItem(item);
    }
    static getJetpackData(id) {
        return this.jetpacks.find((f) => f.jetpackID == id);
    }
}
