import { EquipmentSlot, ItemDurabilityComponent, ItemEnchantableComponent, ItemStack, system, world, } from "@minecraft/server";
import { itemJetpackManager } from "./Managers/Items/itemJetpackManager";
import { blockCustomComponentManager } from "./Managers/CustomComponents/Blocks/blockCustomComponents";
import { playerEquipmentManager } from "./Managers/Player/playerEquipmentManager";
import { bookUIManager } from "./Managers/UI/bookUIManager";
import { playerManager } from "./Managers/Player/playerManager";
system.runInterval(() => {
    itemJetpackManager.initialTick();
}, 10);
world.afterEvents.playerJoin.subscribe((data) => {
    const player = world.getEntity(data.playerId);
    if (!player || player.getDynamicProperty("firstSpawn"))
        return;
    const interval = system.runInterval(() => {
        const velocity = player.getVelocity();
        if (!player || !player.isValid()) {
            system.clearRun(interval);
            return;
        }
        if (velocity.x == 0 && velocity.z == 0)
            return;
        player.playSound("random.pop");
        player.setDynamicProperty("firstSpawn", true);
        playerManager.giveItem(player, new ItemStack("eddsjetpacks:guide_book"));
        player.sendMessage("<§8Edds' Jetpacks§r> You have been given x1 Guide Book");
        system.clearRun(interval);
    });
});
world.afterEvents.itemUse.subscribe((data) => {
    const { itemStack, source } = data;
    if (itemStack.typeId != "eddsjetpacks:guide_book")
        return;
    bookUIManager.show(source, 0);
});
world.afterEvents.itemUse.subscribe((data) => {
    const mainhand = playerEquipmentManager.getSlot(data.source, EquipmentSlot.Mainhand);
    const mainItem = mainhand.getItem();
    if (!mainItem || mainItem.typeId != "eddsjetpacks:oil_bucket")
        return;
    const offhand = playerEquipmentManager.getSlot(data.source, EquipmentSlot.Offhand);
    const offhandItem = offhand.getItem();
    if (!offhandItem)
        return;
    const jetpackData = itemJetpackManager.jetpacks.find((f) => f.emptyJetpack && f.emptyJetpack.itemID == offhandItem.typeId);
    if (!jetpackData)
        return;
    const emptyJetpackData = jetpackData.emptyJetpack;
    if (!emptyJetpackData)
        return;
    data.source.dimension.playSound("bucket.empty_lava", data.source.location);
    const jetpackItem = new ItemStack(jetpackData.jetpackID, 1);
    if (emptyJetpackData.transferDurability) {
        const lastDurComp = offhandItem.getComponent(ItemDurabilityComponent.componentId);
        const newDurComp = jetpackItem.getComponent(ItemDurabilityComponent.componentId);
        newDurComp.damage = lastDurComp.damage;
    }
    if (emptyJetpackData.transferEnchantments) {
        const lastEnchantComp = offhandItem.getComponent(ItemEnchantableComponent.componentId);
        if (lastEnchantComp) {
            const lastEnchants = lastEnchantComp.getEnchantments();
            const newEnchantComp = jetpackItem.getComponent(ItemEnchantableComponent.componentId);
            if (newEnchantComp) {
                newEnchantComp.addEnchantments(lastEnchants);
            }
        }
    }
    if (offhandItem.nameTag) {
        jetpackItem.nameTag = offhandItem.nameTag;
    }
    jetpackItem.setLore(itemJetpackManager.getLore(jetpackData.maxFuel, jetpackData.maxFuel));
    jetpackItem.setDynamicProperty("fuel", jetpackData.maxFuel);
    const bucketItem = new ItemStack("minecraft:bucket", 1);
    offhand.setItem(jetpackItem);
    mainhand.setItem(bucketItem);
});
world.afterEvents.itemStartUse.subscribe((data) => {
    if (data.itemStack.typeId == itemJetpackManager.jetpackItemID) {
        data.source.addTag(itemJetpackManager.usingJetpackTag);
    }
});
world.afterEvents.itemStopUse.subscribe((data) => {
    if (data.itemStack?.typeId == itemJetpackManager.jetpackItemID) {
        data.source.removeTag(itemJetpackManager.usingJetpackTag);
    }
});
world.afterEvents.itemReleaseUse.subscribe((data) => {
    if (data.itemStack?.typeId == itemJetpackManager.jetpackItemID) {
        data.source.removeTag(itemJetpackManager.usingJetpackTag);
    }
});
world.beforeEvents.worldInitialize.subscribe((data) => {
    blockCustomComponentManager.load(data);
});
